'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
	value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _colorsSafe = require('colors/safe');

var _colorsSafe2 = _interopRequireDefault(_colorsSafe);

var _nodeHook = require('./node-hook');

var _nodeHook2 = _interopRequireDefault(_nodeHook);

var _pluginsWriteStats = require('./plugins/write stats');

var _pluginsWriteStats2 = _interopRequireDefault(_pluginsWriteStats);

var _pluginsNotifyStats = require('./plugins/notify stats');

var _pluginsNotifyStats2 = _interopRequireDefault(_pluginsNotifyStats);

// using ES6 template strings
// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/template_strings

var webpack_isomorphic_tools = (function () {
	function webpack_isomorphic_tools(webpack_configuration, options) {
		_classCallCheck(this, webpack_isomorphic_tools);

		this.webpack_configuration = webpack_configuration;
		this.options = options || {};

		// a list of files which can be require()d normally on the server
		// (for example, if you have require("./file.json") both in webpack and in the server code)
		// (should work, not tested)
		options.exceptions = options.exceptions || [];

		if (!options.assets) {
			throw new Error('You must specify "assets" parameter');
		}

		var regular_expressions = {};

		// will be used with write_stats plugin
		// const assets = {}

		// for each user defined asset type
		var _iteratorNormalCompletion = true;
		var _didIteratorError = false;
		var _iteratorError = undefined;

		try {
			for (var _iterator = _getIterator(_Object$keys(this.options.assets)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
				var assets_type = _step.value;

				var description = options.assets[assets_type];

				// create a regular expression for this file type (or these file types)

				var extension_matcher = undefined;
				if (description.extensions && description.extensions.length > 1) {
					extension_matcher = '(' + description.extensions.join('|') + ')';
				} else {
					extension_matcher = description.extension || description.extensions[0];
				}

				regular_expressions[assets_type] = new RegExp('\\.' + extension_matcher + '$');

				// if a "loader" (or "loaders") are specified, then create a webpack module loader
				if (description.loader || description.loaders) {
					var loader = {
						test: regular_expressions[assets_type]
					};

					if (description.loader) {
						loader.loader = description.loader;
					} else {
						loader.loaders = description.loaders;
					}

					if (description.path || description.paths) {
						loader.include = description.paths || [description.path];
					}

					webpack_configuration.module.loaders.push(loader);
				}
			}
		} catch (err) {
			_didIteratorError = true;
			_iteratorError = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion && _iterator['return']) {
					_iterator['return']();
				}
			} finally {
				if (_didIteratorError) {
					throw _iteratorError;
				}
			}
		}

		if (!webpack_configuration.context) {
			throw new Error('You must specify "configuration.context" in your webpack configuration');
		}

		if (!webpack_configuration.output) {
			throw new Error('You must specify "configuration.output" section in your webpack configuration');
		}

		if (!webpack_configuration.output.path) {
			throw new Error('You must specify "configuration.output.path" in your webpack configuration');
		}

		if (!webpack_configuration.output.publicPath) {
			throw new Error('You must specify "configuration.output.publicPath" in your webpack configuration');
		}

		// webpack-stats.json file path
		var webpack_stats_file_path = this.webpack_stats_path();

		// add webpack stats plugins

		// write_stats writes webpack compiled files' names to a special .json file
		// (this will be used later to fetch these files from server)
		function write_stats_plugin() {
			this.plugin('done', function (stats) {
				_pluginsWriteStats2['default'].call(this, stats, {
					environment: options.development ? 'development' : 'production',
					output: webpack_stats_file_path,
					assets: options.assets,
					regular_expressions: regular_expressions
				});
			});
		}

		webpack_configuration.plugins = webpack_configuration.plugins || [];

		if (options.development) {
			webpack_configuration.plugins = webpack_configuration.plugins.concat(
			// outputs stats info to the console
			// (only needed in development mode)
			function () {
				this.plugin('done', _pluginsNotifyStats2['default']);
			}, write_stats_plugin);
		} else {
			webpack_configuration.plugins = webpack_configuration.plugins.concat(write_stats_plugin);
		}
	}

	_createClass(webpack_isomorphic_tools, [{
		key: 'webpack_stats_path',

		// gets webpack-stats.json file path
		value: function webpack_stats_path() {
			return _path2['default'].resolve(this.webpack_configuration.output.path, '..', 'webpack-stats.json');
		}
	}, {
		key: 'assets',

		// returns a mapping to read file paths for all the user specified asset types
		// along with a couple of predefined ones: javascripts and styles
		value: function assets() {
			// webpack and node.js start in parallel
			// so webpack-stats.json might not exist on the very first run
			// (or there should be a better way of webpack notifying about build ending)
			if (!_fs2['default'].existsSync(this.webpack_stats_path())) {
				console.log(_colorsSafe2['default'].red('***** File "' + this.webpack_stats_path() + '" not found. Using an empty stub instead. This is normal because webpack-dev-server and Node.js both start simultaneously and therefore webpack hasn\'t yet finished its build process when Node.js server starts. Just restart your script after Webpack finishes the build (when green letter will appear in the console)'));
				return {};
			}

			var assets = require(this.webpack_stats_path());

			// // invalidate caches with empty data
			// // a better way would be to keep track of all the required modules in require()
			// for (let assets_type of Object.keys(assets))
			// {
			// 	const assets_of_type = assets[assets_type]
			// 	for (let path of Object.keys(assets_of_type))
			// 	{
			// 		const real_path = assets_of_type[path]				
			// 		// http://...
			// 		delete require.cache[require.resolve(real_path)]
			// 	}
			// }

			return assets;
		}
	}, {
		key: 'refresh',

		// clear the require.cache (only used in developer mode with webpack-dev-server)
		value: function refresh() {
			delete require.cache[require.resolve(this.webpack_stats_path())];
			return require(this.webpack_stats_path());
		}
	}, {
		key: 'register',

		// registers a Node.js require hook
		//
		// read this article if you don't know what a "require hook" is
		// http://bahmutov.calepin.co/hooking-into-node-loader-for-fun-and-profit.html
		value: function register() {
			var _this = this;

			// for each user specified asset type which isn't for .js files
			// (because '.js' files requiring already works natively)
			_Object$keys(this.options.assets).filter(function (assets_type) {
				var description = _this.options.assets[assets_type];
				if (description.extension) {
					return description.extension !== 'js';
				} else {
					return description.extensions.indexOf('js') < 0;
				}
			})
			// register a require hook for each file extension of this asset type
			.forEach(function (assets_type) {
				var description = _this.options.assets[assets_type];
				(description.extensions || [description.extension]).forEach(function (extension) {
					_this.register_extension(extension);
				});
			});
		}
	}, {
		key: 'require',

		// is called when you require() your assets
		// (or can be used manually without require hooks)
		value: function require(asset_path) {
			if (!asset_path) {
				return '';
			}

			// console.log('* Requiring', asset_path)

			// get real file path list
			var assets = this.assets();

			// find this asset in the real file path list
			var _iteratorNormalCompletion2 = true;
			var _didIteratorError2 = false;
			var _iteratorError2 = undefined;

			try {
				for (var _iterator2 = _getIterator(_Object$keys(assets)), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
					var type = _step2.value;

					var asset = assets[type][asset_path];
					// if the real path was found in the list - return it
					if (asset) {
						return asset;
					}
				}
			} catch (err) {
				_didIteratorError2 = true;
				_iteratorError2 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion2 && _iterator2['return']) {
						_iterator2['return']();
					}
				} finally {
					if (_didIteratorError2) {
						throw _iteratorError2;
					}
				}
			}

			// serve a not-found asset maybe
			console.log(_colorsSafe2['default'].red('***** Warning. Asset not found: ' + asset_path));
			return '';
		}
	}, {
		key: 'register_extension',

		// registers a require hook for a particular file extension
		value: function register_extension(extension) {
			var _this2 = this;

			_nodeHook2['default'].hook('.' + extension, function (asset_path, fallback) {
				// convert absolute path to relative path
				asset_path = _path2['default'].relative(_this2.webpack_configuration.context, asset_path);

				// convert Windows path to a correct Webpack path
				asset_path = asset_path.replace(/\\/g, '/');
				// add './' in the beginning if it's missing (is the case on Windows for example)
				if (asset_path.indexOf('.') !== 0) {
					asset_path = './' + asset_path;
				}

				// if this filename is in the user specified exceptions list
				// then fallback to the normal require() behaviour
				if (_this2.options.exceptions.indexOf(asset_path) >= 0) {
					return fallback();
				}

				// require() this asset (returns the real file path for this asset, e.g. an image)
				return _this2.require(asset_path);
			});
		}
	}]);

	return webpack_isomorphic_tools;
})();

exports['default'] = webpack_isomorphic_tools;

// a sample path parser of webpack url-loader
// (works for images, fonts, and i guess for everything else, should work for any file type)
webpack_isomorphic_tools.url_loader_path_parser = function (module, resolve_asset_path, options) {
	// retain everything inside of double quotes.
	// usually it's "data:image..." for embedded with the double quotes
	// or __webpack_public_path__ + "..." for filesystem path
	var double_qoute_index = module.source.indexOf('"');
	var asset_path = module.source.slice(double_qoute_index + 1, -1);

	// check if the file was embedded (small enough)
	var is_embedded = asset_path.lastIndexOf('data:image', 0) === 0;
	if (!is_embedded) {
		// if it wasn't embedded then it's a file path so resolve it
		asset_path = resolve_asset_path(asset_path);
	}

	return asset_path;
};
module.exports = exports['default'];
//# sourceMappingURL=index.js.map