'use strict';

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports['default'] = write_stats;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

// writes webpack-stats.json file, which contains assets' file paths

function write_stats(stats, options) {
	var _this = this;

	var _production_ = options.environment === 'production';
	var _development_ = options.environment === 'development';

	var json = stats.toJson();

	var output_path = this.options.output.publicPath;
	var output = {};

	// to see all webpack stats (for debuggin purpose)
	// output.json = json

	var resolve_asset_path = function resolve_asset_path(asset_path) {
		return _this.options.output.publicPath + asset_path;
	};
	// {
	// 	if (_development_)
	// 	{
	// 		// path.resolve doesn't work for Http protocol
	// 		return this.options.output.publicPath + asset_path
	// 	}
	// 	else
	// 	{
	// 		return path.resolve(this.options.output.path, asset_path)
	// 	}
	// }

	output.javascript = {};
	output.styles = {};

	// for each chunk name ("main", "common", ...)
	_Object$keys(json.assetsByChunkName).forEach(function (name) {
		// get javascript chunk real file path

		var javascript = get_assets(name, 'js')[0];
		// the second asset is usually a source map

		if (javascript) {
			output.javascript[name] = javascript;
		}

		// get style chunk real file path

		var style = get_assets(name, 'css')[0];
		// the second asset is usually a source map

		if (style) {
			output.styles[name] = style;
		}
	});

	// gets asset paths by name and extension of their chunk
	function get_assets(name) {
		var extension = arguments.length <= 1 || arguments[1] === undefined ? 'js' : arguments[1];

		var chunk = json.assetsByChunkName[name];

		// a chunk could be a string or an array, so make sure it is an array
		if (!Array.isArray(chunk)) {
			chunk = [chunk];
		}

		return chunk
		// filter by extension
		.filter(function (name) {
			return _path2['default'].extname(name) === '.' + extension;
		})
		// adjust the real path (can be http, filesystem)
		.map(function (name) {
			return resolve_asset_path(name);
		});
	}

	// // output stats for all application javascript entry points
	// Object.keys(this.options.entry).forEach(chunk_name =>
	// {
	// 	let entry = this.options.entry[chunk_name]
	// 	if (Array.isArray(entry))
	// 	{
	// 		entry = entry[entry.length - 1]
	// 	}
	//
	// 	output.javascript[entry] = get_assets(chunk_name, 'js')[0]
	// 	// the second asset is usually a source map
	// })

	// omit node_modules contents and internal webpack modules
	var modules = json.modules.filter(function (module) {
		return module.name.indexOf('./~/') !== 0 && module.name.indexOf('(webpack)') !== 0;
	});

	// one can supply a custom filter
	var default_filter = function default_filter(asset, regular_expression) {
		return regular_expression.test(asset.name);
	};

	// for each user specified asset type
	_Object$keys(options.assets).forEach(function (asset_type) {
		// user settings for this asset type
		var asset_description = options.assets[asset_type];

		// one can supply his own filter
		var filter = (asset_description.filter || default_filter).bind(_this);
		// one can supply his own path_parser
		var path_parser = asset_description.path_parser.bind(_this);

		// path_parser is required
		if (!asset_description.path_parser) {
			throw new Error('path_parser required for asset type "' + asset_type + '"');
		}

		// get real paths for all the files from this asset type
		output[asset_type] = modules
		// take just modules of this asset type
		.filter(function (module) {
			return filter(module, options.regular_expressions[asset_type], options);
		}).reduce(function (set, module) {
			// determine and set the real file path
			set[module.name] = path_parser(module, resolve_asset_path, options);
			return set;
		}, {});
	});

	// for debugging purposes
	// console.log(JSON.stringify(output, null, 2))

	// create all the folders in the path if they don't exist
	_mkdirp2['default'].sync(_path2['default'].dirname(options.output));

	// write webpack stats file
	_fs2['default'].writeFileSync(options.output, JSON.stringify(output));
}

module.exports = exports['default'];
//# sourceMappingURL=write stats.js.map